
(*==========================================================================;
 *
 *  Copyright (C) 1996-1997 Microsoft Corporation.  All Rights Reserved.
 *
 *  File:       dplobby.h
 *  Content:    DirectPlayLobby include file
 *
 ***************************************************************************)

unit DPLobby;

interface

{$Z4}
{$A+}

uses Windows, DPlay;

(*
 * GUIDS used by DirectPlay objects
 *)

const
  CLSID_DirectPlayLobby: TGUID = (D1:$2fe8f810;D2:$b2a5;D3:$11d0;D4:($a7,$87,$0,$0,$f8,$3,$ab,$fc));

  IID_IDirectPlayLobby: TGUID = (D1:$af465c71;D2:$9588;D3:$11cf;D4:($a0,$20,$0,$aa,$0,$61,$57,$ac));
  IID_IDirectPlayLobbyA: TGUID = (D1:$26c66a70;D2:$b367;D3:$11cf;D4:($a0,$24,$0,$aa,$0,$61,$57,$ac));
  IID_IDirectPlayLobby2: TGUID = (D1:$194c220;D2:$a303;D3:$11d0;D4:($9c,$4f,$0,$a0,$c9,$5,$42,$5e));
  IID_IDirectPlayLobby2A: TGUID = (D1:$1bb4af80;D2:$a303;D3:$11d0;D4:($9c,$4f,$0,$a0,$c9,$5,$42,$5e));


(****************************************************************************
 *
 * IDirectPlayLobby Structures
 *
 * Various structures used to invoke DirectPlayLobby.
 *
 ****************************************************************************)


(*
 * DPLAPPINFO
 * Used to hold information about a registered DirectPlay
 * application
 *)
type
  DPLAPPINFO = record
    dwSize: DWORD;            // Size of this structure
    guidApplication: TGUID;   // GUID of the Application
    case Integer of           // Pointer to the Application Name
      0: (lpszAppNameA: LPSTR);
      1: (lpszAppName: LPWSTR);
  end;
  LPDPLAPPINFO = ^DPLAPPINFO;

(*
 * DPCOMPOUNDADDRESSELEMENT
 *
 * An array of these is passed to CreateCompoundAddresses()
 *)
  DPCOMPOUNDADDRESSELEMENT = record
    guidDataType: TGUID;
    dwDataSize: DWORD;
    lpData: Pointer;
  end;
  LPDPCOMPOUNDADDRESSELEMENT = ^DPCOMPOUNDADDRESSELEMENT;

(****************************************************************************
 *
 * Enumeration Method Callback Prototypes
 *
 ****************************************************************************)

(*
 * Callback for EnumAddress()
 *)
  LPDPENUMADDRESSCALLBACK = function(const guidDataType: TGUID;
      dwDataSize: DWORD; lpData: Pointer; lpContext: Pointer): BOOL;
      stdcall;

(*
 * Callback for EnumAddressTypes()
 *)
  LPDPLENUMADDRESSTYPESCALLBACK = function(const guidDataType: TGUID;
      lpContext: Pointer; dwFlags: DWORD): BOOL; stdcall;

(*
 * Callback for EnumLocalApplications()
 *)
  LPDPLENUMLOCALAPPLICATIONSCALLBACK = function(const lpAppInfo: DPLAPPINFO;
      lpContext: Pointer; dwFlags: DWORD): BOOL; stdcall;


(****************************************************************************
 *
 * IDirectPlayLobby (and IDirectPlayLobbyA) Interface
 *
 ****************************************************************************)

type
  IDirectPlayLobby = interface;
  IDirectPlayLobby2 = interface;

  IDirectPlayLobbyA = IDirectPlayLobby;

  IDirectPlayLobby2A = IDirectPlayLobby2;

  IDirectPlayLobby = interface(IUnknown)
    ['{AF465C71-9588-11CF-A020-00AA006157AC}']
    (*** IDirectPlayLobby methods ***)
    function Connect(dwFlags: DWORD; out lplpDP: IDirectPlay2;
        pUnk: IUnknown): HRESULT; stdcall;
    function CreateAddress(const guidSP, guidDataType: TGUID; const lpData;
        dwDataSize: DWORD; var lpAddress; var lpdwAddressSize: DWORD): HRESULT;
        stdcall;
    function EnumAddress(lpEnumAddressCallback: LPDPENUMADDRESSCALLBACK;
        const lpAddress; dwAddressSize: DWORD; lpContext : Pointer): HRESULT;
        stdcall;
    function EnumAddressTypes(lpEnumAddressTypeCallback:
        LPDPLENUMADDRESSTYPESCALLBACK; const guidSP: TGUID; lpContext: Pointer;
        dwFlags: DWORD): HRESULT; stdcall;
    function EnumLocalApplications(lpEnumLocalAppCallback:
        LPDPLENUMLOCALAPPLICATIONSCALLBACK; lpContext: Pointer; dwFlags: DWORD):
        HRESULT; stdcall;
    function GetConnectionSettings(dwAppID: DWORD; var lpData: DPLCONNECTION;
        var lpdwDataSize: DWORD): HRESULT; stdcall;
    function ReceiveLobbyMessage(dwFlags: DWORD; dwAppID: DWORD;
        var lpdwMessageFlags: DWORD; var lpData; var lpdwDataSize: DWORD):
        HRESULT; stdcall;
    function RunApplication(dwFlags: DWORD; var lpdwAppId: DWORD;
        const lpConn: DPLCONNECTION; hReceiveEvent: THandle): HRESULT; stdcall;
    function SendLobbyMessage(dwFlags: DWORD; dwAppID: DWORD; const lpData;
        dwDataSize: DWORD): HRESULT; stdcall;
    function SetConnectionSettings(dwFlags: DWORD; dwAppID: DWORD;
        const lpConn: DPLCONNECTION): HRESULT; stdcall;
    function SetLobbyMessageEvent(dwFlags: DWORD; dwAppID: DWORD;
        hReceiveEvent: THandle): HRESULT; stdcall;
  end;

(****************************************************************************
 *
 * IDirectPlayLobby2 (and IDirectPlayLobby2A) Interface
 *
 ****************************************************************************)
  IDirectPlayLobby2 = interface(IDirectPlayLobby)
    ['{0194C220-A303-11D0-9C4F-00A0C905425E}']
    (*** IDirectPlayLobby methods ***)
    function Connect(dwFlags: DWORD; out lplpDP: IDirectPlay2;
        pUnk: IUnknown): HRESULT; stdcall;
    function CreateAddress(const guidSP, guidDataType: TGUID; const lpData;
        dwDataSize: DWORD; var lpAddress; var lpdwAddressSize: DWORD): HRESULT;
        stdcall;
    function EnumAddress(lpEnumAddressCallback: LPDPENUMADDRESSCALLBACK;
        const lpAddress; dwAddressSize: DWORD; lpContext : Pointer): HRESULT;
        stdcall;
    function EnumAddressTypes(lpEnumAddressTypeCallback:
        LPDPLENUMADDRESSTYPESCALLBACK; const guidSP: TGUID; lpContext: Pointer;
        dwFlags: DWORD): HRESULT; stdcall;
    function EnumLocalApplications(lpEnumLocalAppCallback:
        LPDPLENUMLOCALAPPLICATIONSCALLBACK; lpContext: Pointer; dwFlags: DWORD):
        HRESULT; stdcall;
    function GetConnectionSettings(dwAppID: DWORD; var lpData: DPLCONNECTION;
        var lpdwDataSize: DWORD): HRESULT; stdcall;
    function ReceiveLobbyMessage(dwFlags: DWORD; dwAppID: DWORD;
        var lpdwMessageFlags: DWORD; var lpData; var lpdwDataSize: DWORD):
        HRESULT; stdcall;
    function RunApplication(dwFlags: DWORD; var lpdwAppId: DWORD;
        const lpConn: DPLCONNECTION; hReceiveEvent: THandle): HRESULT; stdcall;
    function SendLobbyMessage(dwFlags: DWORD; dwAppID: DWORD; const lpData;
        dwDataSize: DWORD): HRESULT; stdcall;
    function SetConnectionSettings(dwFlags: DWORD; dwAppID: DWORD;
        const lpConn: DPLCONNECTION): HRESULT; stdcall;
    function SetLobbyMessageEvent(dwFlags: DWORD; dwAppID: DWORD;
        hReceiveEvent: THandle): HRESULT; stdcall;
    (*** IDirectPlayLobby2 methods ***)
    function CreateCompoundAddress(const lpElements: DPCOMPOUNDADDRESSELEMENT;
        dwElementCount: DWORD; var lpAddress; var lpdwAddressSize: DWORD):
        HRESULT; stdcall;
  end;

(****************************************************************************
 *
 * DirectPlayLobby API Prototypes
 *
 ****************************************************************************)

function DirectPlayLobbyCreateW(const lpguidSP: TGUID; out lplpDPL: IDirectPlayLobby;
    lpUnk: IUnknown; lpData: Pointer; dwDataSize: DWORD): HRESULT; stdcall;
function DirectPlayLobbyCreateA(const lpguidSP: TGUID; out lplpDPL: IDirectPlayLobbyA;
    lpUnk: IUnknown; lpData: Pointer; dwDataSize: DWORD): HRESULT; stdcall;
function DirectPlayLobbyCreate(const lpguidSP: TGUID; out lplpDPL: IDirectPlayLobbyA;
    lpUnk: IUnknown; lpData: Pointer; dwDataSize: DWORD): HRESULT; stdcall;


(****************************************************************************
 *
 * DirectPlayLobby Flags
 *
 ****************************************************************************)

(*
 *	This is a message flag used by ReceiveLobbyMessage.  It can be
 *	returned in the dwMessageFlags parameter to indicate a message from
 *	the system.
 *)
const
  DPLMSG_SYSTEM = $00000001;

(*
 *	This is a message flag used by ReceiveLobbyMessage and SendLobbyMessage.
 *  It is used to indicate that the message is a standard lobby message.
 *  DPLMSG_SETPROPERTY, DPLMSG_SETPROPERTYRESPONSE, DPLMSG_GETPROPERTY,
 *	DPLMSG_GETPROPERTYRESPONSE
 *)
  DPLMSG_STANDARD = $00000002;


(****************************************************************************
 *
 * DirectPlayLobby messages and message data structures
 *
 * All system messages have a dwMessageFlags value of DPLMSG_SYSTEM returned
 * from a call to ReceiveLobbyMessage.
 *
 * All standard messages have a dwMessageFlags value of DPLMSG_STANDARD returned
 * from a call to ReceiveLobbyMessage.
 *
 ****************************************************************************)

(*
 * DPLMSG_GENERIC
 * Generic message structure used to identify the message type.
 *)
type
  DPLMSG_GENERIC = record
    dwType: DWORD;   // Message type
  end;
  LPDPLMSG_GENERIC = ^DPLMSG_GENERIC;

(*
 *  DPLMSG_SETPROPERTY
 *  Standard message sent by an application to a lobby to set a
 *  property
 *)
  DPLMSG_SETPROPERTY = record
    dwType: DWORD;                           // Message type
    dwRequestID: DWORD;                      // Request ID (DPL_NOCONFIRMATION if no confirmation desired)
    guidPlayer: TGUID;                       // Player GUID
    guidPropertyTag: TGUID;                  // Property GUID
    dwDataSize: DWORD;                       // Size of data
    dwPropertyData: array[0..0] of DWORD;   // Buffer containing data
  end;
  LPDPLMSG_SETPROPERTY = ^DPLMSG_SETPROPERTY;

const
  DPL_NOCONFIRMATION = 0;

(*
 *  DPLMSG_SETPROPERTYRESPONSE
 *  Standard message returned by a lobby to confirm a 
 *  DPLMSG_SETPROPERTY message.
 *)
type
  DPLMSG_SETPROPERTYRESPONSE = record
    dwType: DWORD;            // Message type
    dwRequestID: DWORD;       // Request ID
    guidPlayer: TGUID;        // Player GUID
    guidPropertyTag: TGUID;   // Property GUID
    hr: HRESULT;              // Return Code
  end;
  LPDPLMSG_SETPROPERTYRESPONSE = ^DPLMSG_SETPROPERTYRESPONSE;

(*
 *  DPLMSG_GETPROPERTY
 *  Standard message sent by an application to a lobby to request
 *	the current value of a property
 *)
  DPLMSG_GETPROPERTY = record
    dwType: DWORD;            // Message type
    dwRequestID: DWORD;       // Request ID
    guidPlayer: TGUID;        // Player GUID
    guidPropertyTag: TGUID;   // Property GUID
  end;
  LPDPLMSG_GETPROPERTY = ^DPLMSG_GETPROPERTY;

(*
 *  DPLMSG_GETPROPERTYRESPONSE
 *  Standard message returned by a lobby in response to a
 *	DPLMSG_GETPROPERTY message.
 *)
  DPLMSG_GETPROPERTYRESPONSE = record
    dwType: DWORD;                           // Message type
    dwRequestID: DWORD;                      // Request ID
    guidPlayer: TGUID;                       // Player GUID
    guidPropertyTag: TGUID;                  // Property GUID
    hr: HRESULT;                             // Return Code
    dwDataSize: DWORD;                       // Size of data
    dwPropertyData: array[0..0] of DWORD;   // Buffer containing data
  end;
  LPDPLMSG_GETPROPERTYRESPONSE = ^DPLMSG_GETPROPERTYRESPONSE;


(******************************************
 *
 *	DirectPlay Lobby message dwType values
 *
 *****************************************)

(*
 *  The application has read the connection settings.
 *  It is now O.K. for the lobby client to release
 *  its IDirectPlayLobby interface.
 *)
const
  DPLSYS_CONNECTIONSETTINGSREAD = $00000001;

(*
 *  The application's call to DirectPlayConnect failed
 *)
  DPLSYS_DPLAYCONNECTFAILED = $00000002;

(*
 *  The application has created a DirectPlay session.
 *)
  DPLSYS_DPLAYCONNECTSUCCEEDED = $00000003;

(*
 *  The application has terminated.
 *)
  DPLSYS_APPTERMINATED = $00000004;

(*
 *  The message is a DPLMSG_SETPROPERTY message.
 *)
  DPLSYS_SETPROPERTY = $00000005;

(*
 *  The message is a DPLMSG_SETPROPERTYRESPONSE message.
 *)
  DPLSYS_SETPROPERTYRESPONSE = $00000006;

(*
 *  The message is a DPLMSG_GETPROPERTY message.
 *)
  DPLSYS_GETPROPERTY = $00000007;

(*
 *  The message is a DPLMSG_GETPROPERTYRESPONSE message.
 *)
  DPLSYS_GETPROPERTYRESPONSE = $00000008;


(****************************************************************************
 *
 * DirectPlay defined property GUIDs and associated data structures
 *
 ****************************************************************************)

(*
 * DPLPROPERTY_MessagesSupported
 *
 * Request whether the lobby supports standard.  Lobby with respond with either
 * TRUE or FALSE or may not respond at all.
 *
 * Property data is a single BOOL with TRUE or FALSE
 *)
  DPLPROPERTY_MessagesSupported: TGUID = (D1:$762ccda1;D2:$d916;D3:$11d0;D4:($ba,$39,$0,$c0,$4f,$d7,$ed,$67));

(*
 * DPLPROPERTY_LobbyGuid
 *
 * Request the GUID that identifies the lobby software that the application
 * is communicating with.
 *
 * Property data is a single GUID.
 *)
  DPLPROPERTY_LobbyGuid: TGUID = (D1:$f56920a0;D2:$d218;D3:$11d0;D4:($ba,$39,$0,$c0,$4f,$d7,$ed,$67));

(*
 * DPLPROPERTY_PlayerGuid
 *
 * Request the GUID that identifies the player on this machine for sending
 * property data back to the lobby.
 *
 * Property data is the DPLDATA_PLAYERDATA structure
 *)
  DPLPROPERTY_PlayerGuid: TGUID = (D1:$b4319322;D2:$d20d;D3:$11d0;D4:($ba,$39,$0,$c0,$4f,$d7,$ed,$67));

(*
 * DPLDATA_PLAYERGUID
 *
 * Data structure to hold the GUID of the player and player creation flags
 * from the lobby.
 *)
type
  DPLDATA_PLAYERGUID = record
    guidPlayer: TGUID;
    dwPlayerFlags: DWORD;   
  end;
  LPDPLDATA_PLAYERGUID = ^DPLDATA_PLAYERGUID;

(*
 * DPLPROPERTY_PlayerScore
 *
 * Used to send an array of long integers to the lobby indicating the 
 * score of a player.
 *
 * Property data is the DPLDATA_PLAYERSCORE structure.
 *)
const
  DPLPROPERTY_PlayerScore: TGUID = (D1:$48784000;D2:$d219;D3:$11d0;D4:($ba,$39,$0,$c0,$4f,$d7,$ed,$67));

(*
 * DPLDATA_PLAYERSCORE
 *
 * Data structure to hold an array of long integers representing a player score.
 * Application must allocate enough memory to hold all the scores.
 *)
type
  DPLDATA_PLAYERSCORE = record
    dwScoreCount: DWORD;
    Score: array[0..0] of Longint;
  end;
  LPDPLDATA_PLAYERSCORE = ^DPLDATA_PLAYERSCORE;

(****************************************************************************
 *
 * DirectPlay Address ID's
 *
 ****************************************************************************)

(* DirectPlay Address
 *
 * A DirectPlay address consists of multiple chunks of data, each tagged
 * with a GUID signifying the type of data in the chunk. The chunk also
 * has a length so that unknown chunk types can be skipped.
 *
 * The EnumAddress() function is used to parse these address data chunks.
 *)

(*
 * DPADDRESS
 *
 * Header for block of address data elements
 *)
  DPADDRESS = record
    guidDataType: TGUID;   
    dwDataSize: DWORD;   
  end;
  LPDPADDRESS = ^DPADDRESS;

(*
 * DPAID_TotalSize
 *
 * Chunk is a DWORD containing size of entire DPADDRESS structure
 *)

const
  DPAID_TotalSize: TGUID = (D1:$1318f560;D2:$912c;D3:$11d0;D4:($9d,$aa,$0,$a0,$c9,$a,$43,$cb));

(*
 * DPAID_ServiceProvider
 *
 * Chunk is a GUID describing the service provider that created the chunk.
 * All addresses must contain this chunk.
 *)

  DPAID_ServiceProvider: TGUID = (D1:$7d916c0;D2:$e0af;D3:$11cf;D4:($9c,$4e,$0,$a0,$c9,$5,$42,$5e));

(*
 * DPAID_LobbyProvider
 *
 * Chunk is a GUID describing the lobby provider that created the chunk.
 * All addresses must contain this chunk.
 *)

  DPAID_LobbyProvider: TGUID = (D1:$59b95640;D2:$9667;D3:$11d0;D4:($a7,$7d,$0,$0,$f8,$3,$ab,$fc));

(*
 * DPAID_Phone and DPAID_PhoneW
 *
 * Chunk is a string containing a phone number (i.e. "1-800-555-1212")
 * in ANSI or UNICODE format
 *)

  DPAID_Phone: TGUID = (D1:$78ec89a0;D2:$e0af;D3:$11cf;D4:($9c,$4e,$0,$a0,$c9,$5,$42,$5e));
  DPAID_PhoneW: TGUID = (D1:$ba5a7a70;D2:$9dbf;D3:$11d0;D4:($9c,$c1,$0,$a0,$c9,$5,$42,$5e));

(*
 * DPAID_Modem and DPAID_ModemW
 *
 * Chunk is a string containing a modem name registered with TAPI
 * in ANSI or UNICODE format
 *)

  DPAID_Modem: TGUID = (D1:$f6dcc200;D2:$a2fe;D3:$11d0;D4:($9c,$4f,$0,$a0,$c9,$5,$42,$5e));

  DPAID_ModemW: TGUID = (D1:$1fd92e0;D2:$a2ff;D3:$11d0;D4:($9c,$4f,$0,$a0,$c9,$5,$42,$5e));

(*
 * DPAID_Inet and DPAID_InetW
 *
 * Chunk is a string containing a TCP/IP host name or an IP address
 * (i.e. "dplay.microsoft.com" or "137.55.100.173") in ANSI or UNICODE format
 *)

  DPAID_INet: TGUID = (D1:$c4a54da0;D2:$e0af;D3:$11cf;D4:($9c,$4e,$0,$a0,$c9,$5,$42,$5e));

  DPAID_INetW: TGUID = (D1:$e63232a0;D2:$9dbf;D3:$11d0;D4:($9c,$c1,$0,$a0,$c9,$5,$42,$5e));

(*
 * DPCOMPORTADDRESS
 *
 * Used to specify com port settings. The constants that define baud rate,
 * stop bits and parity are defined in WINBASE.H. The constants for flow
 * control are given below.
 *)


type
  DPCOMPORTADDRESS = record
    dwComPort: DWORD;       // COM port to use (1-4)
    dwBaudRate: DWORD;      // baud rate (100-256k)
    dwStopBits: DWORD;      // no. stop bits (1-2)
    dwParity: DWORD;        // parity (none, odd, even, mark)
    dwFlowControl: DWORD;   // flow control (none, xon/xoff, rts, dtr)
  end;
  LPDPCOMPORTADDRESS = ^DPCOMPORTADDRESS;

(*
 * DPAID_ComPort
 *
 * Chunk contains a DPCOMPORTADDRESS structure defining the serial port.
 *)

const
  DPAID_ComPort: TGUID = (D1:$f2f0ce00;D2:$e0af;D3:$11cf;D4:($9c,$4e,$0,$a0,$c9,$5,$42,$5e));

(****************************************************************************
 *
 * 	dplobby 1.0 obsolete definitions
 *	Included for compatibility only.
 *
 ****************************************************************************)
  DPLAD_SYSTEM = DPLMSG_SYSTEM;



implementation

function DirectPlayLobbyCreateW; external 'DPLobby.dll';
function DirectPlayLobbyCreateA; external 'DPLobby.dll';
function DirectPlayLobbyCreate; external 'DPLobby.dll' name 'DirectPlayLobbyCreateA';

end.
