
(*==========================================================================;
 *
 *  Copyright (C) 1994-1997 Microsoft Corporation.  All Rights Reserved.
 *
 *  File:       dplay.h
 *  Content:    DirectPlay include file
 *
 ***************************************************************************)

unit DPlay;

interface

{$Z4}
{$A+}

uses Windows;

const
(*
 * GUIDS used by DirectPlay objects
 *)
  CLSID_DirectPlay: TGUID = (D1:$d1eb6d20;D2:$8923;D3:$11d0;D4:($9d,$97,$0,$a0,$c9,$a,$43,$cb));

  IID_IDirectPlay: TGUID = (D1:$5454e9a0;D2:$db65;D3:$11ce;D4:($92,$1c,$00,$aa,$00,$6c,$49,$72));
  
  IID_IDirectPlay2: TGUID = (D1:$2b74f7c0;D2:$9154;D3:$11cf;D4:($a9,$cd,$0,$aa,$0,$68,$86,$e3));
  IID_IDirectPlay2A: TGUID = (D1:$9d460580;D2:$a822;D3:$11cf;D4:($96,$c,$0,$80,$c7,$53,$4e,$82));

  IID_IDirectPlay3: TGUID = (D1:$133efe40;D2:$32dc;D3:$11d0;D4:($9c,$fb,$0,$a0,$c9,$a,$43,$cb));
  IID_IDirectPlay3A: TGUID = (D1:$133efe41;D2:$32dc;D3:$11d0;D4:($9c,$fb,$0,$a0,$c9,$a,$43,$cb));

(*
 * GUIDS used by Service Providers shipped with DirectPlay
 * Use these to identify Service Provider returned by EnumConnections
 *)

  DPSPGUID_IPX: TGUID = (D1:$685bc400;D2:$9d2c;D3:$11cf;D4:($a9,$cd,$0,$aa,$0,$68,$86,$e3));
  DPSPGUID_TCPIP: TGUID = (D1:$36E95EE0;D2:$8577;D3:$11cf;D4:($96,$c,$0,$80,$c7,$53,$4e,$82));
  DPSPGUID_SERIAL: TGUID = (D1:$f1d6860;D2:$88d9;D3:$11cf;D4:($9c,$4e,$0,$a0,$c9,$5,$42,$5e));
  DPSPGUID_MODEM: TGUID = (D1:$44eaa760;D2:$cb68;D3:$11cf;D4:($9c,$4e,$0,$a0,$c9,$5,$42,$5e));

(****************************************************************************
 *
 * DirectPlay Structures
 *
 * Various structures used to invoke DirectPlay.
 *
 ****************************************************************************)

(*
 * DPID
 * DirectPlay player and group ID
 *)
type
  DPID = DWORD;
  LPDPID = ^DWORD;

(*
 * DPID that system messages come from
 *)
const
  DPID_SYSMSG = 0;

(*
 * DPID representing all players in the session
 *)
  DPID_ALLPLAYERS = 0;

(*
 * DPID representing the server player
 *)
  DPID_SERVERPLAYER = 1;

(*
 * The player ID is unknown (used with e.g. DPSESSION_NOMESSAGEID)
 *)
  DPID_UNKNOWN = $FFFFFFFF;

(*
 * DPCAPS
 * Used to obtain the capabilities of a DirectPlay object
 *)
type
  DPCAPS = record
    dwSize: DWORD;              // Size of structure, in bytes
    dwFlags: DWORD;             // DPCAPS_xxx flags
    dwMaxBufferSize: DWORD;     // Maximum message size, in bytes,  for this service provider
    dwMaxQueueSize: DWORD;      // Obsolete.
    dwMaxPlayers: DWORD;        // Maximum players/groups (local + remote)
    dwHundredBaud: DWORD;       // Bandwidth in 100 bits per second units;
                                // i.e. 24 is 2400, 96 is 9600, etc.
    dwLatency: DWORD;           // Estimated latency; 0 = unknown
    dwMaxLocalPlayers: DWORD;   // Maximum # of locally created players allowed
    dwHeaderLength: DWORD;      // Maximum header length, in bytes, on messages
                                // added by the service provider
    dwTimeout: DWORD;           // Service provider's suggested timeout value
                                // This is how long DirectPlay will wait for
                                // responses to system messages
  end;
  LPDPCAPS = ^DPCAPS;

(*
 * This DirectPlay object is the session host.  If the host exits the
 * session, another application will become the host and receive a
 * DPSYS_HOST system message.
 *)
const
  DPCAPS_ISHOST = $00000002;

(*
 * The service provider bound to this DirectPlay object can optimize
 * group messaging.
 *)
  DPCAPS_GROUPOPTIMIZED = $00000008;

(*
 * The service provider bound to this DirectPlay object can optimize
 * keep alives (see DPSESSION_KEEPALIVE)
 *)
  DPCAPS_KEEPALIVEOPTIMIZED = $00000010;

(*
 * The service provider bound to this DirectPlay object can optimize
 * guaranteed message delivery.
 *)
  DPCAPS_GUARANTEEDOPTIMIZED = $00000020;

(*
 * This DirectPlay object supports guaranteed message delivery.
 *)
  DPCAPS_GUARANTEEDSUPPORTED = $00000040;

(*
 * This DirectPlay object supports digital signing of messages.
 *)
  DPCAPS_SIGNINGSUPPORTED = $00000080;

(*
 * This DirectPlay object supports encryption of messages.
 *)
  DPCAPS_ENCRYPTIONSUPPORTED = $00000100;


(*
 * DPSESSIONDESC2
 * Used to describe the properties of a DirectPlay
 * session instance
 *)
type
  DPSESSIONDESC2 = record
    dwSize: DWORD;             // Size of structure
    dwFlags: DWORD;            // DPSESSION_xxx flags
    guidInstance: TGUID;       // ID for the session instance
    guidApplication: TGUID;    // GUID of the DirectPlay application.
                               // GUID_NULL for all applications.
    dwMaxPlayers: DWORD;       // Maximum # players allowed in session
    dwCurrentPlayers: DWORD;   // Current # players in session (read only)
    lpszSessionNameA: PChar;   // HACK - Selected ANSI section from union below
(*
    union
    {                           // Name of the session
        LPWSTR  lpszSessionName;    // Unicode
        LPSTR   lpszSessionNameA;   // ANSI
    };
*)
    lpszPasswordA: PChar;      // HACK - Selected ANSI section from union below
(*
    union
    {                           // Password of the session (optional)
        LPWSTR  lpszPassword;       // Unicode
        LPSTR   lpszPasswordA;      // ANSI
    };
*)
    dwReserved1: DWORD;        // Reserved for future MS use.
    dwReserved2: DWORD;
    dwUser1: DWORD;            // For use by the application
    dwUser2: DWORD;
    dwUser3: DWORD;
    dwUser4: DWORD;
  end;
  LPDPSESSIONDESC2 = ^DPSESSIONDESC2;

(*
 * Applications cannot create new players in this session.
 *)
const
  DPSESSION_NEWPLAYERSDISABLED = $00000001;

(*
 * If the DirectPlay object that created the session, the host,
 * quits, then the host will attempt to migrate to another
 * DirectPlay object so that new players can continue to be created
 * and new applications can join the session.
 *)
  DPSESSION_MIGRATEHOST = $00000004;

(*
 * This flag tells DirectPlay not to set the idPlayerTo and idPlayerFrom
 * fields in player messages.  This cuts two DWORD's off the message
 * overhead.
 *)
  DPSESSION_NOMESSAGEID = $00000008;


(*
 * This flag tells DirectPlay to not allow any new applications to
 * join the session.  Applications already in the session can still
 * create new players.
 *)
  DPSESSION_JOINDISABLED = $00000020;

(*
 * This flag tells DirectPlay to detect when remote players
 * exit abnormally (e.g. their computer or modem gets unplugged)
 *)
  DPSESSION_KEEPALIVE = $00000040;

(*
 * This flag tells DirectPlay not to send a message to all players
 * when a players remote data changes
 *)
  DPSESSION_NODATAMESSAGES = $00000080;

(*
 * This flag indicates that the session belongs to a secure server
 * and needs user authentication
 *)
  DPSESSION_SECURESERVER = $00000100;

(*
 * This flag indicates that the session is private and requirs a password
 * for EnumSessions as well as Open.
 *)
  DPSESSION_PRIVATE = $00000200;

(*
 * This flag indicates that the session requires a password for joining.
 *)
  DPSESSION_PASSWORDREQUIRED = $00000400;

(*
 * This flag tells DirectPlay to route all messages through the server
 *)
  DPSESSION_MULTICASTSERVER = $00000800;

(*
 * This flag tells DirectPlay to only download information about the
 * DPPLAYER_SERVERPLAYER.
 *)
  DPSESSION_CLIENTSERVER = $00001000;

(*
 * DPNAME
 * Used to hold the name of a DirectPlay entity
 * like a player or a group
 *)
type
  DPNAME = record
    dwSize: DWORD;    // Size of structure
    dwFlags: DWORD;   // Not used. Must be zero.
    case Integer of
      // Unicode section
	    0: (lpszShortName: LPWSTR; // The short or friendly name
	        lpszLongName: LPWSTR);
      // ANSI section
      1: (lpszShortNameA: LPSTR; // The long or formal name
          lpszLongNameA: LPSTR);
  end;
  LPDPNAME = ^DPNAME;

(*
 * DPCREDENTIALS
 * Used to hold the user name and password of a DirectPlay user
 *)
  DPCREDENTIALS = record
    dwSize: DWORD;    // Size of structure
    dwFlags: DWORD;   // Not used. Must be zero.
    case Integer of
  		// Unicode section
    	0: (lpszUsername: LPWSTR;   // User name of the account
          lpszPassword: LPWSTR;   // Password of the account
          lpszDomain: LPWSTR);    // Domain name of the account
      // ANSI section
      1: (lpszUsernameA: LPSTR;
          lpszPasswordA: LPSTR;
          lpszDomainA: LPSTR);
  end;
  LPDPCREDENTIALS = ^DPCREDENTIALS;

(*
 * DPSECURITYDESC
 * Used to describe the security properties of a DirectPlay
 * session instance
 *)
  DPSECURITYDESC = record
    dwSize: DWORD;                  // Size of structure
    dwFlags: DWORD;                 // Not used. Must be zero.
    lpszSSIProviderA: LPSTR;        // HACK - Selected ANSI from union below
(*
    union
    {                               // SSPI provider name
        LPWSTR  lpszSSPIProvider;   // Unicode
        LPSTR   lpszSSPIProviderA;  // ANSI
    };
*)
    lpszCAPIProviderA: LPSTR;       // HACK - Selected ANSI from union below
(*
    union
    {                               // CAPI provider name
        LPWSTR lpszCAPIProvider;    // Unicode
        LPSTR  lpszCAPIProviderA;   // ANSI
    };
*)
    dwCAPIProviderType: DWORD;      // Crypto Service Provider type
    dwEncryptionAlgorithm: DWORD;   // Encryption Algorithm type
  end;
  LPDPSECURITYDESC = ^DPSECURITYDESC;

(*
 * DPACCOUNTDESC
 * Used to describe a user membership account
 *)
  DPACCOUNTDESC = record
    dwSize: DWORD;    // Size of structure
    dwFlags: DWORD;   // Not used. Must be zero.
    case Integer of   // Account identifier
    	0: (lpszAccountID: LPWSTR);  // UNICODE
      1: (lpszAccountIDA: LPSTR);  // ANSI
  end;
  LPDPACCOUNTDESC = ^DPACCOUNTDESC;

(*
 * DPLCONNECTION
 * Used to hold all in the informaion needed to connect
 * an application to a session or create a session
 *)
  DPLCONNECTION = record
    dwSize: DWORD;                     // Size of this structure
    dwFlags: DWORD;                    // Flags specific to this structure
    lpSessionDesc: LPDPSESSIONDESC2;   // Pointer to session desc to use on connect
    lpPlayerName: LPDPNAME;            // Pointer to Player name structure
    guidSP: TGUID;                     // GUID of the DPlay SP to use
    lpAddress: Pointer;                // Address for service provider
    dwAddressSize: DWORD;              // Size of address data
  end;
  LPDPLCONNECTION = ^DPLCONNECTION;

(*
 * DPCHAT
 * Used to hold the a DirectPlay chat message
 *)
  DPCHAT = record
    dwSize: DWORD;   
    dwFlags: DWORD;
    case Integer of            // Message string
    	0: (lpszMessage: LPWSTR);  // Unicode
    	1: (lpszMessageA: LPSTR);  // ANSI
  end;
  LPDPCHAT = ^DPCHAT;

(****************************************************************************
 *
 * Prototypes for DirectPlay callback functions
 *
 ****************************************************************************)

(*
 * Callback for IDirectPlay2::EnumSessions
 *)
  LPDPENUMSESSIONSCALLBACK2 = function(const lpThisSD: DPSESSIONDESC2;
      var lpdwTimeOut: DWORD; dwFlags: DWORD; lpContext: Pointer): BOOL;
      stdcall;

(*
 * This flag is set on the EnumSessions callback dwFlags parameter when
 * the time out has occurred. There will be no session data for this
 * callback. If *lpdwTimeOut is set to a non-zero value and the 
 * EnumSessionsCallback function returns TRUE then EnumSessions will 
 * continue waiting until the next timeout occurs. Timeouts are in
 * milliseconds.
 *)
const
  DPESC_TIMEDOUT = $00000001;


(*
 * Callback for IDirectPlay2::EnumPlayers
 *              IDirectPlay2::EnumGroups
 *              IDirectPlay2::EnumGroupPlayers
 *)
type
  LPDPENUMPLAYERSCALLBACK2 = function(dpId: DPID; dwPlayerType: DWORD;
      const lpName: DPNAME; dwFlags: DWORD; lpContext: Pointer): BOOL;
      stdcall;


(*
 * Unicode callback for DirectPlayEnumerate
 * This callback prototype will be used if compiling
 * for Unicode strings
 *)
  LPDPENUMDPCALLBACK = function(const lpguidSP: TGUID; lpSPName: LPWSTR;
      dwMajorVersion: DWORD; dwMinorVersion: DWORD; lpContext: Pointer):
      BOOL; stdcall;

(*
 * ANSI callback for DirectPlayEnumerate
 * This callback prototype will be used if compiling
 * for ANSI strings
 *)
  LPDPENUMDPCALLBACKA = function(const lpguidSP: TGUID; lpSPName: LPSTR;
      dwMajorVersion: DWORD; dwMinorVersion: DWORD; lpContext: Pointer):
      BOOL; stdcall;

(*
 * Callback for IDirectPlay3(A)::EnumConnections
 *)
  LPDPENUMCONNECTIONSCALLBACK = function(const lpguidSP: TGUID;
      lpConnection: Pointer; dwConnectionSize: DWORD; const lpName: DPNAME;
      dwFlags: DWORD; lpContext: Pointer): BOOL; stdcall;


(*
 * API's
 *)
function DirectPlayEnumerateA(lpEnumDPCallback: LPDPENUMDPCALLBACKA;
    lpContext: Pointer): HRESULT; stdcall;
function DirectPlayEnumerateW(lpEnumDPCallback: LPDPENUMDPCALLBACK;
    lpContext: Pointer): HRESULT; stdcall;
function DirectPlayEnumerate(lpEnumDPCallback: LPDPENUMDPCALLBACKA;
    lpContext: Pointer): HRESULT; stdcall;


(****************************************************************************
 *
 * IDirectPlay2 (and IDirectPlay2A) Interface
 *
 ****************************************************************************)
type
  IDirectPlay2 = interface;
  IDirectPlay3 = interface;

  IDirectPlay2A = IDirectPlay2;
  IDirectPlay3A = IDirectPlay3;

  IDirectPlay2 = interface(IUnknown)
    ['{2B74F7C0-9154-11CF-A9CD-00AA006886E3}']
    (*** IDirectPlay2 methods ***)
    function AddPlayerToGroup(idGroup: DPID; idPlayer: DPID): HRESULT; stdcall;
    function Close: HRESULT; stdcall;
    function CreateGroup(var lpidGroup: DPID; var lpGroupName: DPNAME;
        const lpData; dwDataSize: DWORD; dwFlags: DWORD): HRESULT; stdcall;
    function CreatePlayer(var lpidPlayer: DPID; var pPlayerName: DPNAME;
        hEvent: THandle; const lpData; dwDataSize: DWORD; dwFliags: DWORD):
        HRESULT; stdcall;
    function DeletePlayerFromGroup(idGroup: DPID; idPlayer: DPID): HRESULT;
        stdcall;
    function DestroyGroup(idGroup: DPID): HRESULT; stdcall;
    function DestroyPlayer(idPlayer: DPID): HRESULT; stdcall;
    function EnumGroupPlayers(idGroup: DPID; const lpguidInstance: TGUID;
        lpEnumPlayersCallback2: LPDPENUMPLAYERSCALLBACK2; lpContext: Pointer;
        dwFlags: DWORD): HRESULT; stdcall;
    function EnumGroups(const lpguidInstance: TGUID; lpEnumPlayersCallback2:
        LPDPENUMPLAYERSCALLBACK2; lpContext: Pointer; dwFlags: DWORD): HRESULT;
        stdcall;
    function EnumPlayers(const lpguidInstance: TGUID; lpEnumPlayersCallback2:
        LPDPENUMPLAYERSCALLBACK2; lpContext: Pointer; dwFlags: DWORD): HRESULT;
        stdcall;
    function EnumSessions(var lpsd: DPSESSIONDESC2; dwTimeout: DWORD;
        lpEnumSessionsCallback2: LPDPENUMSESSIONSCALLBACK2; lpContext: Pointer;
        dwFlags: DWORD): HRESULT; stdcall;
    function GetCaps(var lpDPCaps: DPCAPS; dwFlags: DWORD): HRESULT; stdcall;
    function GetGroupData(idGroup: DPID; var lpData; var lpdwDataSize: DWORD;
        dwFlags: DWORD): HRESULT; stdcall;
    function GetGroupName(idGroup: DPID; var lpData; var lpdwDataSize: DWORD):
        HRESULT; stdcall;
    function GetMessageCount(idPlayer: DPID; var lpdwCount: DWORD): HRESULT;
         stdcall;
    function GetPlayerAddress(idPlayer: DPID; var lpAddress;
        var lpdwAddressSize: DWORD): HRESULT; stdcall;
    function GetPlayerCaps(idPlayer: DPID; var lpPlayerCaps: DPCAPS;
        dwFlags: DWORD): HRESULT; stdcall;
    function GetPlayerData(idPlayer: DPID; var lpData; var lpdwDataSize: DWORD;
        dwFlags: DWORD): HRESULT; stdcall;
    function GetPlayerName(idPlayer: DPID; var lpData; var lpdwDataSize: DWORD):
        HRESULT; stdcall;
    function GetSessionDesc(var lpData; var lpdwDataSize: DWORD): HRESULT;
         stdcall;
    function Initialize(const lpGUID: TGUID): HRESULT; stdcall;
    function Open(var lpsd: DPSESSIONDESC2; dwFlags: DWORD): HRESULT; stdcall;
    function Receive(var lpidFrom: DPID; var lpidTo: DPID; dwFlags: DWORD;
        var lpData; var lpdwDataSize: DWORD): HRESULT; stdcall;
    function Send(idFrom: DPID; lpidTo: DPID; dwFlags: DWORD; const lpData;
        lpdwDataSize: DWORD): HRESULT; stdcall;
    function SetGroupData(idGroup: DPID; const lpData; dwDataSize: DWORD;
        dwFlags: DWORD): HRESULT; stdcall;
    function SetGroupName(idGroup: DPID; const lpGroupName: DPNAME;
        dwFlags: DWORD): HRESULT; stdcall;
    function SetPlayerData(idPlayer: DPID; const lpData; dwDataSize: DWORD;
        dwFlags: DWORD): HRESULT; stdcall;
    function SetPlayerName(idPlayer: DPID; const lpPlayerName: DPNAME;
        dwFlags: DWORD): HRESULT; stdcall;
    function SetSessionDesc(const lpSessDesc: DPSESSIONDESC2; dwFlags: DWORD):
        HRESULT; stdcall;
  end;

(****************************************************************************
 *
 * IDirectPlay2 interface macros
 *
 ****************************************************************************)


(****************************************************************************
 *
 * IDirectPlay3 (and IDirectPlay3A) Interface
 *
 ****************************************************************************)

  IDirectPlay3 = interface(IDirectPlay2)
    ['{133EFE40-32DC-11D0-9CFB-00A0C90A43CB}']
    (*** IDirectPlay2 methods ***)
    function AddPlayerToGroup(idGroup: DPID; idPlayer: DPID): HRESULT; stdcall;
    function Close: HRESULT; stdcall;
    function CreateGroup(var lpidGroup: DPID; var lpGroupName: DPNAME;
        const lpData; dwDataSize: DWORD; dwFlags: DWORD): HRESULT; stdcall;
    function CreatePlayer(var lpidPlayer: DPID; var pPlayerName: DPNAME;
        hEvent: THandle; const lpData; dwDataSize: DWORD; dwFliags: DWORD):
        HRESULT; stdcall;
    function DeletePlayerFromGroup(idGroup: DPID; idPlayer: DPID): HRESULT;
        stdcall;
    function DestroyGroup(idGroup: DPID): HRESULT; stdcall;
    function DestroyPlayer(idPlayer: DPID): HRESULT; stdcall;
    function EnumGroupPlayers(idGroup: DPID; const lpguidInstance: TGUID;
        lpEnumPlayersCallback2: LPDPENUMPLAYERSCALLBACK2; lpContext: Pointer;
        dwFlags: DWORD): HRESULT; stdcall;
    function EnumGroups(const lpguidInstance: TGUID; lpEnumPlayersCallback2:
        LPDPENUMPLAYERSCALLBACK2; lpContext: Pointer; dwFlags: DWORD): HRESULT;
        stdcall;
    function EnumPlayers(const lpguidInstance: TGUID; lpEnumPlayersCallback2:
        LPDPENUMPLAYERSCALLBACK2; lpContext: Pointer; dwFlags: DWORD): HRESULT;
        stdcall;
    function EnumSessions(var lpsd: DPSESSIONDESC2; dwTimeout: DWORD;
        lpEnumSessionsCallback2: LPDPENUMSESSIONSCALLBACK2; lpContext: Pointer;
        dwFlags: DWORD): HRESULT; stdcall;
    function GetCaps(var lpDPCaps: DPCAPS; dwFlags: DWORD): HRESULT; stdcall;
    function GetGroupData(idGroup: DPID; var lpData; var lpdwDataSize: DWORD;
        dwFlags: DWORD): HRESULT; stdcall;
    function GetGroupName(idGroup: DPID; var lpData; var lpdwDataSize: DWORD):
        HRESULT; stdcall;
    function GetMessageCount(idPlayer: DPID; var lpdwCount: DWORD): HRESULT;
         stdcall;
    function GetPlayerAddress(idPlayer: DPID; var lpAddress;
        var lpdwAddressSize: DWORD): HRESULT; stdcall;
    function GetPlayerCaps(idPlayer: DPID; var lpPlayerCaps: DPCAPS;
        dwFlags: DWORD): HRESULT; stdcall;
    function GetPlayerData(idPlayer: DPID; var lpData; var lpdwDataSize: DWORD;
        dwFlags: DWORD): HRESULT; stdcall;
    function GetPlayerName(idPlayer: DPID; var lpData; var lpdwDataSize: DWORD):
        HRESULT; stdcall;
    function GetSessionDesc(var lpData; var lpdwDataSize: DWORD): HRESULT;
         stdcall;
    function Initialize(const lpGUID: TGUID): HRESULT; stdcall;
    function Open(var lpsd: DPSESSIONDESC2; dwFlags: DWORD): HRESULT; stdcall;
    function Receive(var lpidFrom: DPID; var lpidTo: DPID; dwFlags: DWORD;
        var lpData; var lpdwDataSize: DWORD): HRESULT; stdcall;
    function Send(idFrom: DPID; lpidTo: DPID; dwFlags: DWORD; const lpData;
        lpdwDataSize: DWORD): HRESULT; stdcall;
    function SetGroupData(idGroup: DPID; const lpData; dwDataSize: DWORD;
        dwFlags: DWORD): HRESULT; stdcall;
    function SetGroupName(idGroup: DPID; const lpGroupName: DPNAME;
        dwFlags: DWORD): HRESULT; stdcall;
    function SetPlayerData(idPlayer: DPID; const lpData; dwDataSize: DWORD;
        dwFlags: DWORD): HRESULT; stdcall;
    function SetPlayerName(idPlayer: DPID; const lpPlayerName: DPNAME;
        dwFlags: DWORD): HRESULT; stdcall;
    function SetSessionDesc(const lpSessDesc: DPSESSIONDESC2; dwFlags: DWORD):
        HRESULT; stdcall;
    (*** IDirectPlay3 methods ***)
    function AddGroupToGroup(idParentGroup: DPID; idGroup: DPID): HRESULT;
        stdcall;
    function CreateGroupInGroup(idParentGroup: DPID; var lpidGroup: DPID;
        var lpGroupName: DPNAME; const lpData; dwDataSize: DWORD;
        dwFlags: DWORD): HRESULT; stdcall;
    function DeleteGroupFromGroup(idParentGroup: DPID; idGroup: DPID): HRESULT;
         stdcall;
    function EnumConnections(const lpguidApplication: TGUID;
        lpEnumCallback: LPDPENUMCONNECTIONSCALLBACK; lpContext: Pointer;
        dwFlags: DWORD): HRESULT; stdcall;
    function EnumGroupsInGroup(idGroup: DPID; const lpguidInstance: TGUID;
        lpEnumPlayersCallback2: LPDPENUMPLAYERSCALLBACK2; lpContext: Pointer;
        dwFlags: DWORD): HRESULT; stdcall;
    function GetGroupConnectionSettings(dwFlags: DWORD; idGroup: DPID;
        var lpData; var lpdwDataSize: DWORD): HRESULT; stdcall;
    function InitializeConnection(lpConnection: Pointer; dwFlags: DWORD):
         HRESULT; stdcall;
    function SecureOpen(const lpsd: DPSESSIONDESC2; dwFlags: DWORD;
        const lpSecurity: DPSECURITYDESC; const lpCredentials: DPCREDENTIALS):
        HRESULT; stdcall;
    function SendChatMessage(idFrom: DPID; idTo: DPID; dwFlags: DWORD;
        const lpChatMessage: DPCHAT): HRESULT; stdcall;
    function SetGroupConnectionSettings(dwFlags: DWORD; idGroup: DPID;
        const lpConnection: DPLCONNECTION): HRESULT; stdcall;
    function StartSession(dwFlags: DWORD; idGroup: DPID): HRESULT; stdcall;
    function GetGroupFlags(idGroup: DPID; var lpdwFlags: DWORD): HRESULT;
        stdcall;
    function GetGroupParent(idGroup: DPID; var lpidParent: DPID): HRESULT;
        stdcall;
    function GetPlayerAccount(idPlayer: DPID; dwFlags: DWORD; var lpData;
        var lpdwDataSize: DWORD): HRESULT; stdcall;
    function GetPlayerFlags(idPlayer: DPID; var lpdwFlags: DWORD): HRESULT;
         stdcall;
  end;

(****************************************************************************
 *
 * IDirectPlay3 interface macros
 *
 ****************************************************************************)


(****************************************************************************
 *
 * EnumConnections API flags
 *
 ****************************************************************************)

(*
 * Enumerate Service Providers
 *)
const
  DPCONNECTION_DIRECTPLAY = $00000001;

(*
 * Enumerate Lobby Providers
 *)
  DPCONNECTION_DIRECTPLAYLOBBY = $00000002;


(****************************************************************************
 *
 * EnumPlayers API flags
 *
 ****************************************************************************)

(*
 * Enumerate all players in the current session
 *)
  DPENUMPLAYERS_ALL = $00000000;
  DPENUMGROUPS_ALL = DPENUMPLAYERS_ALL;


(*
 * Enumerate only local (created by this application) players
 * or groups
 *)
  DPENUMPLAYERS_LOCAL = $00000008;
  DPENUMGROUPS_LOCAL = DPENUMPLAYERS_LOCAL;

(*
 * Enumerate only remote (non-local) players
 * or groups
 *)
  DPENUMPLAYERS_REMOTE = $00000010;
  DPENUMGROUPS_REMOTE = DPENUMPLAYERS_REMOTE;

(*
 * Enumerate groups along with the players
 *)
  DPENUMPLAYERS_GROUP = $00000020;

(*
 * Enumerate players or groups in another session 
 * (must supply lpguidInstance)
 *)
  DPENUMPLAYERS_SESSION = $00000080;
  DPENUMGROUPS_SESSION = DPENUMPLAYERS_SESSION;

(*
 * Enumerate server players
 *)
  DPENUMPLAYERS_SERVERPLAYER = $00000100;

(*
 * Enumerate spectator players
 *)
  DPENUMPLAYERS_SPECTATOR = $00000200;

(*
 * Enumerate shortcut groups
 *)
  DPENUMGROUPS_SHORTCUT = $00000400;

(*
 * Enumerate staging area groups
 *)
  DPENUMGROUPS_STAGINGAREA = $00000800;

(****************************************************************************
 *
 * CreatePlayer API flags
 *
 ****************************************************************************)

(*
 * This flag indicates that this player should be designated
 * the server player. The app should specify this at CreatePlayer.
 *)
  DPPLAYER_SERVERPLAYER = DPENUMPLAYERS_SERVERPLAYER;

(*
 * This flag indicates that this player should be designated
 * a spectator. The app should specify this at CreatePlayer.
 *)
  DPPLAYER_SPECTATOR = DPENUMPLAYERS_SPECTATOR;

(*
 * This flag indicates that this player was created locally.
 * (returned from GetPlayerFlags)
 *)
  DPPLAYER_LOCAL = DPENUMPLAYERS_LOCAL;

(****************************************************************************
 *
 * CreateGroup API flags
 *
 ****************************************************************************)


(*
 * This flag indicates that the StartSession can be called on the group.
 * The app should specify this at CreateGroup, or CreateGroupInGroup.
 *)
  DPGROUP_STAGINGAREA = DPENUMGROUPS_STAGINGAREA;

(*
 * This flag indicates that this group was created locally.
 * (returned from GetGroupFlags)
 *)
  DPGROUP_LOCAL = DPENUMGROUPS_LOCAL;

(****************************************************************************
 *
 * EnumSessions API flags
 *
 ****************************************************************************)

(*
 * Enumerate sessions which can be joined
 *)
  DPENUMSESSIONS_AVAILABLE = $00000001;

(*
 * Enumerate all sessions even if they can't be joined.
 *)
  DPENUMSESSIONS_ALL = $00000002;




(*
 * Start an asynchronous enum sessions
 *)
  DPENUMSESSIONS_ASYNC = $00000010;

(*
 * Stop an asynchronous enum sessions
 *)
  DPENUMSESSIONS_STOPASYNC = $00000020;
 
(*
 * Enumerate sessions even if they require a password
 *)
  DPENUMSESSIONS_PASSWORDREQUIRED = $00000040;

(*
 * Return status about progress of enumeration instead of
 * showing any status dialogs.
 *)
  DPENUMSESSIONS_RETURNSTATUS = $00000080;

(****************************************************************************
 *
 * GetCaps and GetPlayerCaps API flags
 *
 ****************************************************************************)

(*
 * The latency returned should be for guaranteed message sending.
 * Default is non-guaranteed messaging.
 *)
  DPGETCAPS_GUARANTEED = $00000001;

 
(****************************************************************************
 *
 * GetGroupData, GetPlayerData API flags
 * Remote and local Group/Player data is maintained separately.
 * Default is DPGET_REMOTE.
 *
 ****************************************************************************)

(*
 * Get the remote data (set by any DirectPlay object in
 * the session using DPSET_REMOTE)
 *)
  DPGET_REMOTE = $00000000;

(*
 * Get the local data (set by this DirectPlay object 
 * using DPSET_LOCAL)
 *)
  DPGET_LOCAL = $00000001;


(****************************************************************************
 *
 * Open API flags
 *
 ****************************************************************************)

(*
 * Join the session that is described by the DPSESSIONDESC2 structure
 *)
  DPOPEN_JOIN = $00000001;

(*
 * Create a new session as described by the DPSESSIONDESC2 structure
 *)
  DPOPEN_CREATE = $00000002;

(*
 * Return status about progress of open instead of showing
 * any status dialogs.
 *)
  DPOPEN_RETURNSTATUS = DPENUMSESSIONS_RETURNSTATUS;

(****************************************************************************
 *
 * DPLCONNECTION flags
 *
 ****************************************************************************)

(*
 * This application should create a new session as
 * described by the DPSESIONDESC structure
 *)
  DPLCONNECTION_CREATESESSION = DPOPEN_CREATE;

(*
 * This application should join the session described by 
 * the DPSESIONDESC structure with the lpAddress data
 *)
  DPLCONNECTION_JOINSESSION = DPOPEN_JOIN;

(****************************************************************************
 *
 * Receive API flags
 * Default is DPRECEIVE_ALL
 *
 ****************************************************************************)

(*
 * Get the first message in the queue
 *)
  DPRECEIVE_ALL = $00000001;

(*
 * Get the first message in the queue directed to a specific player 
 *)
  DPRECEIVE_TOPLAYER = $00000002;

(*
 * Get the first message in the queue from a specific player
 *)
  DPRECEIVE_FROMPLAYER = $00000004;

(*
 * Get the message but don't remove it from the queue
 *)
  DPRECEIVE_PEEK = $00000008;


(****************************************************************************
 *
 * Send API flags
 *
 ****************************************************************************)

(*
 * Send the message using a guaranteed send method.
 * Default is non-guaranteed.
 *)
  DPSEND_GUARANTEED = $00000001;


(*
 * This flag is obsolete. It is ignored by DirectPlay
 *)
  DPSEND_HIGHPRIORITY = $00000002;

(*
 * This flag is obsolete. It is ignored by DirectPlay
 *)
  DPSEND_OPENSTREAM = $00000008;

(*
 * This flag is obsolete. It is ignored by DirectPlay
 *)
  DPSEND_CLOSESTREAM = $00000010;

(*
 * Send the message digitally signed to ensure authenticity.
 *)
  DPSEND_SIGNED = $00000020;

(*
 * Send the message with encryption to ensure privacy.
 *)
  DPSEND_ENCRYPTED = $00000040;


(****************************************************************************
 *
 * SetGroupData, SetGroupName, SetPlayerData, SetPlayerName,
 * SetSessionDesc API flags.
 * Default is DPSET_REMOTE.
 *
 ****************************************************************************)

(*
 * Propagate the data to all players in the session
 *)
  DPSET_REMOTE = $00000000;

(*
 * Do not propagate the data to other players
 *)
  DPSET_LOCAL = $00000001;

(*
 * Used with DPSET_REMOTE, use guaranteed message send to
 * propagate the data
 *)
  DPSET_GUARANTEED = $00000002;


(****************************************************************************
 *
 * DirectPlay system messages and message data structures
 *
 * All system message come 'From' player DPID_SYSMSG.  To determine what type 
 * of message it is, cast the lpData from Receive to DPMSG_GENERIC and check
 * the dwType member against one of the following DPSYS_xxx constants. Once
 * a match is found, cast the lpData to the corresponding of the DPMSG_xxx
 * structures to access the data of the message.
 *
 ****************************************************************************)

(*
 * A new player or group has been created in the session
 * Use DPMSG_CREATEPLAYERORGROUP.  Check dwPlayerType to see if it
 * is a player or a group.
 *)
  DPSYS_CREATEPLAYERORGROUP = $0003;

(*
 * A player has been deleted from the session
 * Use DPMSG_DESTROYPLAYERORGROUP
 *)
  DPSYS_DESTROYPLAYERORGROUP = $0005;

(*
 * A player has been added to a group
 * Use DPMSG_ADDPLAYERTOGROUP
 *)
  DPSYS_ADDPLAYERTOGROUP = $0007;

(*
 * A player has been removed from a group
 * Use DPMSG_DELETEPLAYERFROMGROUP
 *)
  DPSYS_DELETEPLAYERFROMGROUP = $0021;

(*
 * This DirectPlay object lost its connection with all the
 * other players in the session.
 * Use DPMSG_SESSIONLOST.
 *)
  DPSYS_SESSIONLOST = $0031;

(*
 * The current host has left the session.
 * This DirectPlay object is now the host.
 * Use DPMSG_HOST.
 *)
  DPSYS_HOST = $0101;

(*
 * The remote data associated with a player or
 * group has changed. Check dwPlayerType to see
 * if it is a player or a group
 * Use DPMSG_SETPLAYERORGROUPDATA
 *)
  DPSYS_SETPLAYERORGROUPDATA = $0102;

(*
 * The name of a player or group has changed.
 * Check dwPlayerType to see if it is a player
 * or a group.
 * Use DPMSG_SETPLAYERORGROUPNAME
 *)
  DPSYS_SETPLAYERORGROUPNAME = $0103;

(*
 * The session description has changed.
 * Use DPMSG_SETSESSIONDESC
 *)
  DPSYS_SETSESSIONDESC = $0104;

(*
 * A group has been added to a group
 * Use DPMSG_ADDGROUPTOGROUP
 *)
  DPSYS_ADDGROUPTOGROUP = $0105;

(*
 * A group has been removed from a group
 * Use DPMSG_DELETEGROUPFROMGROUP
 *)
  DPSYS_DELETEGROUPFROMGROUP = $0106;

(*
 * A secure player-player message has arrived.
 * Use DPMSG_SECUREMESSAGE
 *)
  DPSYS_SECUREMESSAGE = $0107;

(*
 * Start a new session.
 * Use DPMSG_STARTSESSION
 *)
  DPSYS_STARTSESSION = $0108;

(*
 * A chat message has arrived
 * Use DPMSG_CHAT
 *)
  DPSYS_CHAT = $0109;

(*
 * Used in the dwPlayerType field to indicate if it applies to a group
 * or a player
 *)
  DPPLAYERTYPE_GROUP = $00000000;
  DPPLAYERTYPE_PLAYER = $00000001;


(*
 * DPMSG_GENERIC
 * Generic message structure used to identify the message type.
 *)
type
  DPMSG_GENERIC = record
    dwType: DWORD;   // Message type
  end;
  LPDPMSG_GENERIC = ^DPMSG_GENERIC;

(*
 * DPMSG_CREATEPLAYERORGROUP
 * System message generated when a new player or group
 * created in the session with information about it.
 *)
  DPMSG_CREATEPLAYERORGROUP = record
    dwType: DWORD;             // Message type
    dwPlayerType: DWORD;       // Is it a player or group
    dpId: DPID;                // ID of the player or group
    dwCurrentPlayers: DWORD;   // current # players & groups in session
    lpData: Pointer;           // pointer to remote data
    dwDataSize: DWORD;         // size of remote data
    dpnName: DPNAME;           // structure with name info
                               // the following fields are only available when using
                               // the IDirectPlay3 interface or greater
    dpIdParent: DPID;          // id of parent group
    dwFlags: DWORD;            // player or group flags
  end;
  LPDPMSG_CREATEPLAYERORGROUP = ^DPMSG_CREATEPLAYERORGROUP;

(*
 * DPMSG_DESTROYPLAYERORGROUP
 * System message generated when a player or group is being
 * destroyed in the session with information about it.
 *)
  DPMSG_DESTROYPLAYERORGROUP = record
    dwType: DWORD;             // Message type
    dwPlayerType: DWORD;       // Is it a player or group
    dpId: DPID;                // player ID being deleted
    lpLocalData: Pointer;      // copy of players local data
    dwLocalDataSize: DWORD;    // sizeof local data
    lpRemoteData: Pointer;     // copy of players remote data
    dwRemoteDataSize: DWORD;   // sizeof remote data
                               // the following fields are only available when using
                               // the IDirectPlay3 interface or greater
    dpnName: DPNAME;           // structure with name info
    dpIdParent: DPID;          // id of parent group
    dwFlags: DWORD;            // player or group flags
  end;
  LPDPMSG_DESTROYPLAYERORGROUP = ^DPMSG_DESTROYPLAYERORGROUP;

(*
 * DPMSG_ADDPLAYERTOGROUP
 * System message generated when a player is being added
 * to a group.
 *)
  DPMSG_ADDPLAYERTOGROUP = record
    dwType: DWORD;      // Message type
    dpIdGroup: DPID;    // group ID being added to
    dpIdPlayer: DPID;   // player ID being added
  end;
  LPDPMSG_ADDPLAYERTOGROUP = ^DPMSG_ADDPLAYERTOGROUP;

(*
 * DPMSG_DELETEPLAYERFROMGROUP
 * System message generated when a player is being
 * removed from a group
 *)
  DPMSG_DELETEPLAYERFROMGROUP = DPMSG_ADDPLAYERTOGROUP;
  LPDPMSG_DELETEPLAYERFROMGROUP = ^DPMSG_DELETEPLAYERFROMGROUP;

(*
 * DPMSG_ADDGROUPTOGROUP
 * System message generated when a group is being added
 * to a group.
 *)
  DPMSG_ADDGROUPTOGROUP = record
    dwType: DWORD;           // Message type
    dpIdParentGroup: DPID;   // group ID being added to
    dpIdGroup: DPID;         // group ID being added
  end;
  LPDPMSG_ADDGROUPTOGROUP = ^DPMSG_ADDGROUPTOGROUP;

(*
 * DPMSG_DELETEGROUPFROMGROUP
 * System message generated when a GROUP is being
 * removed from a group
 *)
  DPMSG_DELETEGROUPFROMGROUP = DPMSG_ADDGROUPTOGROUP;
  LPDPMSG_DELETEGROUPFROMGROUP = ^DPMSG_DELETEGROUPFROMGROUP;

(*
 * DPMSG_SETPLAYERORGROUPDATA
 * System message generated when remote data for a player or
 * group has changed.
 *)
  DPMSG_SETPLAYERORGROUPDATA = record
    dwType: DWORD;         // Message type
    dwPlayerType: DWORD;   // Is it a player or group
    dpId: DPID;            // ID of player or group
    lpData: Pointer;       // pointer to remote data
    dwDataSize: DWORD;     // size of remote data
  end;
  LPDPMSG_SETPLAYERORGROUPDATA = ^DPMSG_SETPLAYERORGROUPDATA;

(*
 * DPMSG_SETPLAYERORGROUPNAME
 * System message generated when the name of a player or
 * group has changed.
 *)
  DPMSG_SETPLAYERORGROUPNAME = record
    dwType: DWORD;         // Message type
    dwPlayerType: DWORD;   // Is it a player or group
    dpId: DPID;            // ID of player or group
    dpnName: DPNAME;       // structure with new name info
  end;
  LPDPMSG_SETPLAYERORGROUPNAME = ^DPMSG_SETPLAYERORGROUPNAME;

(*
 * DPMSG_SETSESSIONDESC
 * System message generated when session desc has changed
 *)
  DPMSG_SETSESSIONDESC = record
    dwType: DWORD;            // Message type
    dpDesc: DPSESSIONDESC2;   // Session desc
  end;
  LPDPMSG_SETSESSIONDESC = ^DPMSG_SETSESSIONDESC;

(*
 * DPMSG_HOST
 * System message generated when the host has migrated to this
 * DirectPlay object.
 *
 *)
  DPMSG_HOST = DPMSG_GENERIC;
  LPDPMSG_HOST = ^DPMSG_HOST;

(*
 * DPMSG_SESSIONLOST
 * System message generated when the connection to the session is lost.
 *
 *)
  DPMSG_SESSIONLOST = DPMSG_GENERIC;
  LPDPMSG_SESSIONLOST = ^DPMSG_SESSIONLOST;

(*
 * DPMSG_SECUREMESSAGE
 * System message generated when a player requests a secure send
 *)
  DPMSG_SECUREMESSAGE = record
    dwType: DWORD;       // Message Type
    dwFlags: DWORD;      // Signed/Encrypted
    dpIdFrom: DPID;      // ID of Sending Player
    lpData: Pointer;     // Player message
    dwDataSize: DWORD;   // Size of player message
  end;
  LPDPMSG_SECUREMESSAGE = ^DPMSG_SECUREMESSAGE;

(*
 * DPMSG_STARTSESSION
 * System message containing all information required to
 * start a new session
 *)
  DPMSG_STARTSESSION = record
    dwType: DWORD;             // Message type
    lpConn: LPDPLCONNECTION;   // DPLCONNECTION structure
  end;
  LPDPMSG_STARTSESSION = ^DPMSG_STARTSESSION;

(*
 * DPMSG_CHAT
 * System message containing a chat message
 *)
  DPMSG_CHAT = record
    dwType: DWORD;        // Message type
    dwFlags: DWORD;       // Message flags
    idFromPlayer: DPID;   // ID of the Sending Player
    idToPlayer: DPID;     // ID of the To Player
    idToGroup: DPID;      // ID of the To Group
    lpChat: LPDPCHAT;     // Pointer to a structure containing the chat message
  end;
  LPDPMSG_CHAT = ^DPMSG_CHAT;

(****************************************************************************
 *
 * DIRECTPLAY ERRORS
 *
 * Errors are represented by negative values and cannot be combined.
 *
 ****************************************************************************)
const
  DP_OK = S_OK;
  DPERR_ALREADYINITIALIZED = $88770000 + 5;
  DPERR_ACCESSDENIED = $88770000 + 10;
  DPERR_ACTIVEPLAYERS = $88770000 + 20;
  DPERR_BUFFERTOOSMALL = $88770000 + 30;
  DPERR_CANTADDPLAYER = $88770000 + 40;
  DPERR_CANTCREATEGROUP = $88770000 + 50;
  DPERR_CANTCREATEPLAYER = $88770000 + 60;
  DPERR_CANTCREATESESSION = $88770000 + 70;
  DPERR_CAPSNOTAVAILABLEYET = $88770000 + 80;
  DPERR_EXCEPTION = $88770000 + 90;
  DPERR_GENERIC = E_FAIL;
  DPERR_INVALIDFLAGS = $88770000 + 120;
  DPERR_INVALIDOBJECT = $88770000 + 130;
  DPERR_INVALIDPARAM = E_INVALIDARG;
  DPERR_INVALIDPARAMS = DPERR_INVALIDPARAM;
  DPERR_INVALIDPLAYER = $88770000 + 150;
  DPERR_INVALIDGROUP = $88770000 + 155;
  DPERR_NOCAPS = $88770000 + 160;
  DPERR_NOCONNECTION = $88770000 + 170;
  DPERR_NOMEMORY = E_OUTOFMEMORY;
  DPERR_OUTOFMEMORY = DPERR_NOMEMORY;
  DPERR_NOMESSAGES = $88770000 + 190;
  DPERR_NONAMESERVERFOUND = $88770000 + 200;
  DPERR_NOPLAYERS = $88770000 + 210;
  DPERR_NOSESSIONS = $88770000 + 220;
  E_PENDING = $80070007;  // HACK - Should be defined in DInput instead
  DPERR_PENDING = E_PENDING;
  DPERR_SENDTOOBIG = $88770000 + 230;
  DPERR_TIMEOUT = $88770000 + 240;
  DPERR_UNAVAILABLE = $88770000 + 250;
  DPERR_UNSUPPORTED = E_NOTIMPL;
  DPERR_BUSY = $88770000 + 270;
  DPERR_USERCANCEL = $88770000 + 280;
  DPERR_NOINTERFACE = E_NOINTERFACE;
  DPERR_CANNOTCREATESERVER = $88770000 + 290;
  DPERR_PLAYERLOST = $88770000 + 300;
  DPERR_SESSIONLOST = $88770000 + 310;
  DPERR_UNINITIALIZED = $88770000 + 320;
  DPERR_NONEWPLAYERS = $88770000 + 330;
  DPERR_INVALIDPASSWORD = $88770000 + 340;
  DPERR_CONNECTING = $88770000 + 350;


  DPERR_BUFFERTOOLARGE = $88770000 + 1000;
  DPERR_CANTCREATEPROCESS = $88770000 + 1010;
  DPERR_APPNOTSTARTED = $88770000 + 1020;
  DPERR_INVALIDINTERFACE = $88770000 + 1030;
  DPERR_NOSERVICEPROVIDER = $88770000 + 1040;
  DPERR_UNKNOWNAPPLICATION = $88770000 + 1050;
  DPERR_NOTLOBBIED = $88770000 + 1070;
  DPERR_SERVICEPROVIDERLOADED = $88770000 + 1080;
  DPERR_ALREADYREGISTERED = $88770000 + 1090;
  DPERR_NOTREGISTERED = $88770000 + 1100;

//
// Security related errors
//
  DPERR_AUTHENTICATIONFAILED = $88770000 + 2000;
  DPERR_CANTLOADSSPI = $88770000 + 2010;
  DPERR_ENCRYPTIONFAILED = $88770000 + 2020;
  DPERR_SIGNFAILED = $88770000 + 2030;
  DPERR_CANTLOADSECURITYPACKAGE = $88770000 + 2040;
  DPERR_ENCRYPTIONNOTSUPPORTED = $88770000 + 2050;
  DPERR_CANTLOADCAPI = $88770000 + 2060;
  DPERR_NOTLOGGEDIN = $88770000 + 2070;
  DPERR_LOGONDENIED = $88770000 + 2080;


(****************************************************************************
 *
 * 	dplay 1.0 obsolete structures + interfaces
 *	Included for compatibility only. New apps should
 *	use IDirectPlay2
 *
 ****************************************************************************)

// define this to ignore obsolete interfaces and constants

  DPOPEN_OPENSESSION = DPOPEN_JOIN;
  DPOPEN_CREATESESSION = DPOPEN_CREATE;

  DPENUMSESSIONS_PREVIOUS = $00000004;

  DPENUMPLAYERS_PREVIOUS = $00000004;

  DPSEND_GUARANTEE = DPSEND_GUARANTEED;
  DPSEND_TRYONCE = $00000004;

  DPCAPS_NAMESERVICE = $00000001;
  DPCAPS_NAMESERVER = DPCAPS_ISHOST;
  DPCAPS_GUARANTEED = $00000004;

  DPLONGNAMELEN = 52;
  DPSHORTNAMELEN = 20;
  DPSESSIONNAMELEN = 32;
  DPPASSWORDLEN = 16;
  DPUSERRESERVED = 16;

  DPSYS_ADDPLAYER = $0003;
  DPSYS_DELETEPLAYER = $0005;

  DPSYS_DELETEGROUP = $0020;
  DPSYS_DELETEPLAYERFROMGRP = $0021;
  DPSYS_CONNECT = $484b;

type
  DPMSG_ADDPLAYER = record
    dwType: DWORD;   
    dwPlayerType: DWORD;
    dpId: DPID;
    szLongName: array[0..DPLONGNAMELEN-1] of Char;
    szShortName: array[0..DPSHORTNAMELEN-1] of Char;
    dwCurrentPlayers: DWORD;
  end;
  LPDPMSG_ADDPLAYER = ^DPMSG_ADDPLAYER;

  DPMSG_ADDGROUP = DPMSG_ADDPLAYER;

  DPMSG_GROUPADD = record
    dwType: DWORD;
    dpIdGroup: DPID;
    dpIdPlayer: DPID;
  end;
  LPDPMSG_GROUPADD = ^DPMSG_GROUPADD;

  DPMSG_GROUPDELETE = DPMSG_GROUPADD;
  DPMSG_DELETEPLAYER = record
    dwType: DWORD;
    dpId: DPID;
  end;
  LPDPMSG_DELETEPLAYER = ^DPMSG_DELETEPLAYER;

  LPDPENUMPLAYERSCALLBACK = function(dpId: DPID; lpFriendlyName: LPSTR;
      lpFormalName: LPSTR; dwFlags: DWORD; lpContext: Pointer): BOOL;
      stdcall;

  DPSESSIONDESC = record
    dwSize: DWORD;
    guidSession: TGUID;
    dwSession: DWORD;
    dwMaxPlayers: DWORD;
    dwCurrentPlayers: DWORD;
    dwFlags: DWORD;
    szSessionName: Array [0..DPSESSIONNAMELEN-1] of char;
    szUserField: Array [0..DPUSERRESERVED-1] of char;
    dwReserved1: DWORD;
    szPassword: Array [0..DPPASSWORDLEN-1] of char;
    dwReserved2: DWORD;
    dwUser1: DWORD;
    dwUser2: DWORD;
    dwUser3: DWORD;
    dwUser4: DWORD;
  end;
  LPDPSESSIONDESC = ^DPSESSIONDESC;

  LPDPENUMSESSIONSCALLBACK = function(var lpDPSessionDesc: DPSESSIONDESC;
      lpContext: Pointer; var lpdwTimeOut: DWORD; dwFlags: DWORD): BOOL;
      stdcall;

type
(*
 * IDirectPlay
 *)
  IDirectPlay = interface(IUnknown)
    (*** IDirectPlay methods ***)
    function AddPlayerToGroup(pidGroup: DPID; pidPlayer: DPID): HRESULT;
        stdcall;
    function Close: HRESULT; stdcall;
    function CreatePlayer(var lppidID: DPID; lpPlayerFriendlyName: LPSTR;
        lpPlayerFormalName: LPSTR; lpEvent: PHandle): HRESULT; stdcall;
    function CreateGroup(var lppidID: DPID; lpGroupFriendlyName: LPSTR;
        lpGroupFormalName: LPSTR): HRESULT; stdcall;
    function DeletePlayerFromGroup(pidGroup: DPID; pidPlayer: DPID): HRESULT;
        stdcall;
    function DestroyPlayer(pidID: DPID): HRESULT; stdcall;
    function DestroyGroup(pidID: DPID): HRESULT; stdcall;
    function EnableNewPlayers(bEnable: BOOL): HRESULT; stdcall;
    function EnumGroupPlayers(pidGroupPID: DPID; lpEnumPlayersCallback:
        LPDPENUMPLAYERSCALLBACK; lpContext: Pointer; dwFlags: DWORD): HRESULT;
        stdcall;
    function EnumGroups(dwSessionID: DWORD; lpEnumPlayersCallback:
        LPDPENUMPLAYERSCALLBACK; lpContext: Pointer; dwFlags: DWORD): HRESULT;
        stdcall;
    function EnumPlayers(dwSessionId: DWORD; lpEnumPlayersCallback:
        LPDPENUMPLAYERSCALLBACK; lpContext: Pointer; dwFlags: DWORD): HRESULT;
        stdcall;
    function EnumSessions(const lpSDesc: DPSESSIONDESC; dwTimeout: DWORD;
        lpEnumSessionsCallback: LPDPENUMSESSIONSCALLBACK; lpContext: Pointer;
        dwFlags: DWORD): HRESULT; stdcall;
    function GetCaps(const lpDPCaps: DPCAPS): HRESULT; stdcall;
    function GetMessageCount(pidID: DPID; var lpdwCount: DWORD): HRESULT;
        stdcall;
    function GetPlayerCaps(pidID: DPID; const lpDPPlayerCaps: DPCAPS): HRESULT;
        stdcall;
    function GetPlayerName(pidID: DPID; lpPlayerFriendlyName: LPSTR;
        var lpdwFriendlyNameLength: DWORD; lpPlayerFormalName: LPSTR;
        var lpdwFormalNameLength: DWORD): HRESULT; stdcall;
    function Initialize(const lpGUID: TGUID): HRESULT; stdcall;
    function Open(const lpSDesc: DPSESSIONDESC): HRESULT; stdcall;
    function Receive(var lppidFrom, lppidTo: DPID; dwFlags: DWORD;
        var lpvBuffer; var lpdwSize: DWORD): HRESULT; stdcall;
    function SaveSession(lpSessionName: LPSTR): HRESULT; stdcall;
    function Send(pidFrom: DPID; pidTo: DPID; dwFlags: DWORD;
        const lpvBuffer; dwBuffSize: DWORD): HRESULT; stdcall;
    function SetPlayerName(pidID: DPID; lpPlayerFriendlyName: LPSTR;
        lpPlayerFormalName: LPSTR): HRESULT; stdcall;
  end;

(*
 *  API's (cont.)
 *)
function DirectPlayCreate(const lpGUID: TGUID; out lplpDP: IDirectPlay;
    pUnk: IUnknown): HRESULT; stdcall;

implementation

function DirectPlayEnumerateA; external 'DPlayX.dll';
function DirectPlayEnumerateW; external 'DPlayX.dll';
function DirectPlayEnumerate; external 'DPlayX.dll' name 'DirectPlayEnumerateA';

function DirectPlayCreate; external 'DPlayX.dll';

end.
