unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DirectSound, WaveEngine,
  Menus, Buttons, StdCtrls, ExtCtrls, ComCtrls;

type
  TForm1 = class(TForm)
    Output: TDsOutput;
    Buf: TDsBuffer;
    OpenDialog: TOpenDialog;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Open1: TMenuItem;
    Exit1: TMenuItem;
    DriversMenu: TMenuItem;
    PageControl: TPageControl;
    PlaySheet: TTabSheet;
    RecordSheet: TTabSheet;
    Label1: TLabel;
    LoopBtn: TSpeedButton;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    PlayBtn: TBitBtn;
    StopBtn: TBitBtn;
    NotifyPanel: TPanel;
    VolTrack: TTrackBar;
    PanTrack: TTrackBar;
    FreqTrack: TTrackBar;
    ThreeDCheck: TCheckBox;
    XTrack: TTrackBar;
    YTrack: TTrackBar;
    ZTrack: TTrackBar;
    Capture: TDsCapture;
    WaveLibrary: TDsWaves;
    CapBuf: TDsCaptureBuffer;
    RecBtn: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure PlayBtnClick(Sender: TObject);
    procedure StopBtnClick(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure LoopBtnClick(Sender: TObject);
    procedure BufNotificationsNotify(Sender: TObject);
    procedure VolTrackChange(Sender: TObject);
    procedure PanTrackChange(Sender: TObject);
    procedure FreqTrackChange(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure ThreeDCheckClick(Sender: TObject);
    procedure XTrackChange(Sender: TObject);
    procedure YTrackChange(Sender: TObject);
    procedure ZTrackChange(Sender: TObject);
    procedure DriversMenuClick(Sender: TObject);
    procedure RecBtnClick(Sender: TObject);
  private
    FFilename: string;
    procedure ReloadBuffer;
  public
    Player: TStreamedPlayer;
    procedure UpdateTracks;
    procedure LoadFile(const Filename: string);
  end;

var Form1: TForm1;

implementation

uses DDraw, MMSystem;

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
var i: integer;
    Item: TMenuItem;
begin
  for i := 0 to DsDrivers.Count-1 do
  begin
    Item := TMenuItem.Create(Self);
    Item.Caption := DsDrivers[i];
    Item.Tag := i;
    Item.OnClick := DriversMenuClick;
    Item.Checked := Output.GUID = PGUID(DsDrivers.Objects[i]);
    DriversMenu.Add(Item);
  end;

  WaveLibrary.BaseDir := ExtractFilePath(Application.ExeName);
  OpenDialog.InitialDir := WaveLibrary.BaseDir;
  UpdateTracks;

  Output.Enabled := True;
  LoadFile(ExtractFilePath(Application.Exename) + 'Drum.wav');
end;

procedure TForm1.DriversMenuClick(Sender: TObject);
var i: integer;
    Item: TMenuItem;
begin
  Output.GUID := PGUID(DsDrivers.Objects[(Sender as TMenuItem).Tag]);
  for i := 0 to DriversMenu.Count-1 do
  begin
    Item := DriversMenu.Items[i];
    Item.Checked := Output.GUID = PGUID(DsDrivers.Objects[i]);
  end;
  ReloadBuffer;
  UpdateTracks;
end;

procedure TForm1.UpdateTracks;
begin
  VolTrack.Position := Buf.Volume;
  PanTrack.Position := Buf.Pan;
  FreqTrack.Position := Buf.Frequency;
  XTrack.Position := Round(Buf.Buffer3D.Position.X);
  YTrack.Position := Round(Buf.Buffer3D.Position.Y);
  ZTrack.Position := Round(Buf.Buffer3D.Position.Z);
end;

// Commented lines in the following procedures show
// how to use streaming for playing [large] files thru
// a smaller buffer

procedure TForm1.PlayBtnClick(Sender: TObject);
begin
//  Player.Play;
  Buf.Play;
end;

procedure TForm1.StopBtnClick(Sender: TObject);
begin
//  Player.Stop;
  Buf.Stop;
end;

procedure TForm1.LoopBtnClick(Sender: TObject);
begin
//  if Assigned(Player) then Player.Looped := LoopBtn.Down;
  Buf.Looped := LoopBtn.Down;
end;

procedure TForm1.LoadFile(const Filename: string);
begin
  Buf.Stop;
//  Player.Free;
//  Player := TStreamedPlayer.Create(Buf, TFileWaveStream.Create(Filename));
  Buf.LoadFromFile(Filename);
  FFilename := Filename;
  Buf.Notifications[0].Offset := (Buf.Length shr 2 - 4) and (-4);
  Buf.Notifications[1].Offset := (Buf.Length shr 1 - 4) and (-4);
  UpdateTracks;
  PlayBtn.Enabled := True;
  LoopBtnClick(Self);
end;

procedure TForm1.ReloadBuffer;
begin
  if FFilename <> '' then LoadFile(FFilename);
end;

procedure TForm1.Open1Click(Sender: TObject);
begin
  with OpenDialog do if Execute then
    LoadFile(Filename);
end;

procedure TForm1.BufNotificationsNotify(Sender: TObject);
begin
  NotifyPanel.Color := TDsBufferNotification(Sender).Tag;
end;

procedure TForm1.VolTrackChange(Sender: TObject);
begin
  Buf.Volume := VolTrack.Position;
end;

procedure TForm1.PanTrackChange(Sender: TObject);
begin
  Buf.Pan := PanTrack.Position;
end;

procedure TForm1.FreqTrackChange(Sender: TObject);
begin
  Buf.Frequency := FreqTrack.Position;
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.ThreeDCheckClick(Sender: TObject);
begin
  with Buf do if ThreeDCheck.Checked
    then Options := Options + [dbo3D]
    else Options := Options - [dbo3D];
  ReloadBuffer;
  UpdateTracks;
end;

procedure TForm1.XTrackChange(Sender: TObject);
begin
  Buf.Buffer3D.Position.X := XTrack.Position;
end;

procedure TForm1.YTrackChange(Sender: TObject);
begin
  Buf.Buffer3D.Position.Y := YTrack.Position;
end;

procedure TForm1.ZTrackChange(Sender: TObject);
begin
  Buf.Buffer3D.Position.Z := ZTrack.Position;
end;

procedure TForm1.RecBtnClick(Sender: TObject);
var Data: PChar;
begin
  Data := nil;
  Screen.Cursor := crHourGlass;
  try
    Output.Enabled := False;
    Capture.Enabled := True;
    try
      with CapBuf do
      begin
        Start;
        Notifications[0].Event.WaitFor(10000);
        Data := AllocMem(Length);
        Lock(0, Length);
        CopyMemory(Data, Data1, Data1Len);
        if Data2Len > 0 then
          CopyMemory(Data + Data1Len, Data2, Data2Len);
        Unlock;
      end;
    finally
      Capture.Enabled := False;
      Output.Enabled := True;
    end;
    with Buf do
    begin
      Format := CapBuf.Format;
      Length := CapBuf.Length;
      Lock(0, Length);
      CopyMemory(Data1, Data, Data1Len);
      if Data2Len > 0 then
        CopyMemory(Data2, Data + Data1Len, Data2Len);
      Unlock;
    end;
    PageControl.ActivePage := PlaySheet;
  finally
    FreeMem(Data);
    Screen.Cursor := crDefault;
  end;
end;

end.
